@@ -1,14 +1,19 @@
 from datetime import datetime, timedelta
 from plone.memoize import ram
 from Products.CMFCore.utils import getToolByName
-from zope.app.component.hooks import getSite
 from zope.globalrequest import getRequest
 from zope.i18n import translate
 import cgi
 import os.path
 
+try:
+    from zope.component.hooks import getSite
+except ImportError:
+    # plone 4.0 support
+    from zope.app.component.hooks import getSite
 
-def link(icon=True, tooltip=False, classes=None, attrs=None):
+
+def link(icon=True, tooltip=False, classes=None, attrs=None, icon_only=False):
     """Generates a helper.
 
     Attributes:
@@ -41,7 +46,8 @@ def link(icon=True, tooltip=False, classes=None, attrs=None):
                 attributes['class'].append('rollover')
                 attributes['title'] = cgi.escape(description, quote=True)
 
-        return linked(item, value, show_icon=icon, attrs=attributes)
+        return linked(item, value, show_icon=icon, attrs=attributes,
+                      icon_only=icon_only)
     return _helper
 
 
@@ -150,7 +156,7 @@ def readable_date(item, date):
         return None
 
 
-def linked(item, value, show_icon=True, attrs=None):
+def linked(item, value, show_icon=True, attrs=None, icon_only=False):
     if attrs is None:
         attrs = {}
 
@@ -202,7 +208,8 @@ def linked(item, value, show_icon=True, attrs=None):
         if item.portal_type in types_using_view:
             href = os.path.join(href, 'view')
 
-    attrs['href'] = href
+    if 'href' not in attrs:
+        attrs['href'] = href
 
     if attrs['class']:
         attrs['class'] = ' '.join(sorted(set(attrs['class'])))
@@ -212,6 +219,8 @@ def linked(item, value, show_icon=True, attrs=None):
         ['%s="%s"' % (attrkey, attrvalue) for attrkey, attrvalue in
          sorted(attrs.items())])
 
+    if icon_only:
+        value = ''
     link = u'<a %s>%s%s</a>' % (attrs_str, img, value)
     wrapper = u'<span class="linkWrapper">%s</span>' % link
     return wrapper
