@@ -1,7 +1,6 @@
 from zope import interface
 from zope import schema
-from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
-from zope.app.component import hooks
+from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from zope.i18nmessageid.message import Message
 from zope.i18n import translate
 from ftw.table.column import METADATA, FIELD, COLUMN
@@ -13,6 +12,12 @@ try:
 except ImportError:
     import simplejson as json
 
+try:
+    from zope.component import hooks
+except ImportError:
+    # plone 4.0 support
+    from zope.app.component import hooks
+
 
 class TableGenerator(object):
     """ generates a html table. See README.txt for usage"""
@@ -125,7 +130,11 @@ class TableGenerator(object):
                                                   context=self.request)
                     else:
                         col['header'] = column['title']
+
                     col['id'] = key
+                    col['sortable'] = column.get('sortable', True)
+                    col['hidden'] = column.get('hidden', False)
+
                     if not column['title']:
                         if key == 'draggable':
                             col['menuDisabled'] = False
@@ -133,12 +142,14 @@ class TableGenerator(object):
                             col['sortable'] = True
                         else:
                             col['menuDisabled'] = True
-                        col['width'] = 30
+                        col['width'] = column.get('width', 30)
                         col['hideable'] = False
                         col['resizable'] = False
                         col['fixed'] = True
                     else:
-                        col['sortable'] = True
+                        if column.get('width', None):
+                            col['width'] = column['width']
+
                     meta_data['fields'].append(field)
                     meta_data['columns'].append(col)
 
@@ -246,8 +257,13 @@ class TableGenerator(object):
         return processed_columns
 
     def process_column(self, column):
+        width = None
         attr = sort_index = title = u""
         transform = lambda x, y: y
+
+        sortable = True
+        hidden = False
+
         if isinstance(column, basestring):
             attr = sort_index = column
 
@@ -276,6 +292,9 @@ class TableGenerator(object):
             title = column.get('column_title', title)
             sort_index = column.get('sort_index', sort_index)
             transform = column.get('transform', transform)
+            width = column.get('width', None)
+            sortable = column.get('sortable', True)
+            hidden = column.get('hidden', False)
 
         title = len(title) and title or attr
         sort_index = len(sort_index) and sort_index or attr
@@ -284,4 +303,7 @@ class TableGenerator(object):
         return {'attr': attr,
                 'title': title,
                 'sort_index': sort_index,
-                'transform': transform}
+                'transform': transform,
+                'width': width,
+                'sortable': sortable,
+                'hidden': hidden}
